/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvSignalVar.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_E2Cld4m1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dyuUbwCu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_C6xJ3LaC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5Eh5N03A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oq2Oc1ZF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_JQfOXzI7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fTew6kV6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_a12IRoJn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VElWxfD2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GzpTF53v_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZYPVWLGb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Og2sSrp9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_GBqjNf8r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VJdIq37l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_R83Nrnra_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gHWZkEEW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_Cxu3GAHN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Rh57vPMN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Trxi9IvX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_lsdffKnh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_9o5SwfuX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kfXRiDKr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Ac1etujd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_pPBLTY6s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_EN7WlLdt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_lcSHFand_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_EpZjyQ1u;
  SignalLine sLine_NblwnY7t;
  SignalLine sLine_4yqUHnHm;
  ControlBinop cBinop_E2Cld4m1;
  ControlBinop cBinop_dyuUbwCu;
  ControlBinop cBinop_C6xJ3LaC;
  ControlBinop cBinop_5Eh5N03A;
  ControlBinop cBinop_fTew6kV6;
  ControlBinop cBinop_VElWxfD2;
  ControlBinop cBinop_GzpTF53v;
  ControlBinop cBinop_ZYPVWLGb;
  ControlVar cVar_Og2sSrp9;
  ControlVar cVar_GBqjNf8r;
  SignalVarf sVarf_e22WApdB;
  SignalVarf sVarf_2Is2fCiR;
  SignalVarf sVarf_1u7tfD9K;
  SignalVarf sVarf_cEui3znk;
  ControlBinop cBinop_Rh57vPMN;
  ControlBinop cBinop_kfXRiDKr;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
